require 'fluent/plugin/output'
require 'fluent/plugin/prometheus'

module Fluent::Plugin
  class PrometheusOutputMetric < Fluent::Plugin::Output
    Fluent::Plugin.register_output('prometheus_metric', self)
    include Fluent::Plugin::PrometheusLabelParser

    def initialize
      super
      @registry = ::Prometheus::Client.registry
    end

    def multi_workers_ready?
      true
    end

    config_param :key, :string

    def process(tag, es)
      # Write out values in event stream to Registry
      es.each do |time, record|
        # Create metric if not exists
        begin
          desc = "fluentd message stream with tag: ${tag}"
          gauge = @registry.gauge(tag.to_sym, desc.sub!("${tag}", tag))
        rescue ::Prometheus::Client::Registry::AlreadyRegisteredError
          gauge = @registry.get(tag.to_sym)
        end
        value = record[@key]
        if value
          record.tap do |hs| 
            hs.delete(@key)
            hs.delete('timestamp')
          end
          # set labels as all elements in message besides the key
          gauge.set(record.transform_keys(&:to_sym), value)
        end
      end
    end

  end
end
